
# q-expansion of newform 6006.2.a.b, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6006
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [-4], [-1], [1], [1], [-6], [4], [4], [-10], [-10], [0], [6], [2], [-8], [10], [10], [-2], [14], [-10], [10], [4], [4], [10], [12], [-12], [8], [0], [10], [10], [-16], [-10], [12], [-4], [22], [16], [-14], [-6], [-8], [-8], [-12], [-10], [8], [-18], [-18], [-20], [-22], [-2], [20], [14], [-12], [4], [-10], [-20], [14], [18], [-22], [8], [24], [-30], [24], [-30], [-20], [-24], [6], [-6], [-2], [2], [24], [2], [-10], [0], [0], [-4], [-22], [-4], [-18], [-6], [-8], [-22], [-12], [28], [32], [26], [-10], [28], [24], [10], [-10], [-36], [20], [24], [24], [-16], [-22], [24], [-4], [-18], [-20], [-2], [-26], [18], [-18], [20], [-10], [12], [6], [30], [-24], [36], [30], [-34], [-40], [28], [-4], [-30], [28], [24], [22], [-4], [-2], [26], [-12], [-24], [-48], [-18], [-16], [24], [-32], [-14], [-16], [16], [8], [-6], [-30], [-50], [20], [-28], [2], [-40], [-4], [10], [8], [-44], [-50], [12], [-38], [30], [20], [-14], [-22], [-18], [-52], [-52], [-24], [8], [52], [-34], [-52], [6], [-48], [-56], [48], [-48], [8], [-4], [56], [-42], [2], [-30], [52], [26], [6], [-18], [-44], [2], [60], [10], [-2], [14], [52], [-46], [58], [-6], [18], [-34], [-44], [8], [-26], [-32], [-46], [36], [28], [60], [-48], [54], [-50], [-18], [2], [32], [14], [-6], [50], [-28], [26], [50], [2], [-64], [18], [-36], [-6], [30], [56], [54], [48], [40], [4], [66], [38], [-6], [40], [-20], [-60], [-52], [-66], [-56], [-6], [-12], [-46], [12], [-14], [-8], [-16], [64], [28], [0], [28], [-10], [4], [40], [64], [-36], [-32], [-28], [-14], [36], [14], [26], [-12], [74], [-50], [-14], [58], [10], [16], [4], [-50], [44], [74], [-26], [48], [0], [-26], [-42], [-38], [12], [-82], [30], [64], [-22], [18], [62], [64], [-64], [52], [-54], [-12], [-10], [26], [-10], [-48], [4], [-48], [22], [-60], [72], [-2], [2], [34], [-30], [2], [32], [-18], [-12], [-44], [32], [40], [78], [-22], [-28], [52], [-66], [-64], [4], [16], [-74], [-36], [50], [46], [-64], [12], [54], [-48], [4], [48], [-12], [-50], [-24], [84], [-12], [-76], [50], [52], [-56], [70], [-70], [-34], [-12], [6], [-30], [-74], [-66], [14], [-40], [-68], [54], [-12], [38], [-32], [34], [16], [-90], [18], [-16], [-58], [-48], [-22], [22], [0], [-42], [-28], [26], [-14], [-6], [-16], [-66], [-64], [4], [24], [-54], [-14], [50], [-12], [12], [12], [-22], [-42], [64], [46], [68], [36], [8], [-78], [42], [-46], [6], [-30], [6], [34], [-56], [-4], [-44], [-72], [18], [-44], [40], [32], [26], [12], [48], [-38], [-76], [-4], [28], [10], [-26], [-20], [-66], [-16], [-34], [-78], [30], [32], [6], [46], [46], [-32], [-66], [56], [54], [-24], [98], [20], [42], [50], [-94], [28], [100], [18], [-4], [2], [66], [38], [74], [-68], [-76], [76], [54], [-52], [-12]]
